﻿--[[

************************************************************************

AckisRecipeList
$Date: 2008-10-02 18:03:29 +0000 (Thu, 02 Oct 2008) $
$Rev: 897 $

Author: Ackis on Illidan US Horde
GUI done by Zhinjio

************************************************************************

Please see Wowace.com for more information.

Documentation:
	Please see Documentation.txt for comprehensive documentation.

************************************************************************

--]]


AckisRecipeList = LibStub("AceAddon-3.0"):NewAddon("Ackis Recipe List", "AceConsole-3.0", "AceEvent-3.0")

local MODNAME			= "Ackis Recipe List"

local addon				= LibStub("AceAddon-3.0"):GetAddon(MODNAME)

local BFAC				= LibStub("LibBabble-Faction-3.0"):GetLookupTable()
local L					= LibStub("AceLocale-3.0"):GetLocale(MODNAME)


-- We should probably clean thes up and use arguments/return values
-- Global variables which are used between multiple files
addon.SkillType = nil

addon.playerProfession = ""
addon.MaxFilterIndex = 129

addon.optionsFrame = {}

-- Frame variables
addon.ScanButton = nil
addon.Frame = nil

-- Make global API calls local to speed things up
local GetNumTradeSkills = GetNumTradeSkills
local GetSpellInfo = GetSpellInfo
local GetSpellName = GetSpellName
local GetTradeSkillLine = GetTradeSkillLine
local GetTradeSkillInfo = GetTradeSkillInfo

local select = select
local format = format
local string = string
local tostring = tostring
local tonumber = tonumber
local pairs = pairs
local table = table
local next = next
local InterfaceOptionsFrame_OpenToFrame = InterfaceOptionsFrame_OpenToFrame
local BOOKTYPE_SPELL = BOOKTYPE_SPELL

-- Constants which are used everytime the add-on is loaded

-- New Shiny addonversion without Revision
addon.addonversion = GetAddOnMetadata("AckisRecipeList", "Version")

-- Global constants which are used between multiple files
addon.ARLTitle = "ARL (v." .. addon.addonversion .. ")"
addon.FullTitle = "Ackis Recipe List (v." .. addon.addonversion .. ")"

-- We use line breaks quite often lets make it a constant send you can't use \t in WoW
addon.br = "\n    - " -- Nuke this

--[[

	Initialization functions

]]--

-- Description: Function run when the addon is initialized.  Registers the slash commands, options, and database
-- Expected result: The addon parameters are initialized, slash commands are enabled, database is enabled, and
--	interface options are registered
-- Input: None
-- Output: None

function addon:OnInitialize()

	self.db = LibStub("AceDB-3.0"):New("ARLDB2")

	self:SetupOptions()

	-- Register slash commands
	self:RegisterChatCommand("arl", "ChatCommand")
	self:RegisterChatCommand("ackisrecipelist", "ChatCommand")

	-- Set default options, which are to include everything in the scan
	self.db:RegisterDefaults(
	{
		profile = {
			-- Sorting Options
			sorting = L["Skill"],

			-- Display Options
			includefiltered = false,
			closeguionskillclose = false,

			-- Filter Options
			filters = {
				general = {			-- General Filters
				    faction = true,
					class = false,
					specialty = false,
					skill = true,
					known = false,
					unknown = true,
				},
				obtain = {			-- Obtain Options
					trainer = true,
					vendor = true,
					instance = true,
					raid = true,
					seasonal = true,
					quest = true,
					pvp = true,
					discovery = true,
					worlddrop = true,
					mobdrop = true,
				},
				item = {			-- Item Filters (Armor/Weapon)
					armor = {
						cloth = true,
						leather = true,
						mail = true,
						plate = true,
						trinket = true,
						cloak = true,
						ring = true,
						necklace = true,
					},
					weapon = {
						onehand = true,
						twohand = true,
						axe = true,
						sword = true,
						mace = true,
						polearm = true,
						dagger = true,
						fist = true,
						staff = true,
						wand = true,
						thrown = true,
						bow = true,
						crossbow = true,
						ammo = true,
					},
				},
				binding = {
					itemboe = true,
					itembop = true,
					recipebop = true,
					recipeboe = true,
				},
				player = {
					melee = true,
					tank = true,
					healer = true,
					caster = true,
				},
				rep = {			-- Reputation Options
					aldor = true,
					scryer = true,
					argentdawn = true,
					ashtonguedeathsworn = true,
					cenarioncircle = true,
					cenarionexpedition = true,
					consortium = true,
					hellfire = true,
					keepersoftime = true,
					nagrand = true,
					lowercity = true,
					scaleofthesands = true,
					shatar = true,
					shatteredsun = true,
					sporeggar = true,
					thoriumbrotherhood = true,
					timbermaw = true,
					violeteye = true,
					zandalar = true,
-- Below this is WotLK stuff. May need to be changed post beta.
					argentcrusade = true,
					frenzyheart = true,
					ebonblade = true,
					kirintor = true,
					sonsofhodir = true,
					kaluak = true,
					oracles = true,
					wyrmrest = true,
					silvercovenant = true,
					sunreavers = true,
					explorersleague = true,
					valiance = true,
					handofvengeance = true,
					taunka = true,
					warsongoffensive = true,
				}
			}
		}
	} )

end

-- Description: Function run when the addon is enabled.  Registers events and pre-loads certain variables.
-- Expected result: Events are registered, scan button and faction levels are identified
-- Input: None
-- Output: None

function addon:OnEnable()

	-- Make addon respond to the tradeskill windows being shown
	self:RegisterEvent("TRADE_SKILL_SHOW")

	-- Addon responds to tradeskill windows being closed.
	self:RegisterEvent("TRADE_SKILL_CLOSE")

	-- Addon responds when a players faction level changes
	self:RegisterEvent("UPDATE_FACTION")

	-- Add an option so that ARL will work with Manufac
	if (Manufac) then
		Manufac.options.args.ARLScan = {
			type = 'execute',
			name = L["Scan Skills"],
			desc = L["Scan Skills Long"],
			func = function() addon:AckisRecipeList_Command() end,
			order = 550,
		}
	end

	-- Populate the repuatation level
	self:GetFactionLevels()

	--Create the button now for later use
	self:CreateScanButton()

end

-- Description: Run when the addon is disabled. Ace3 takes care of unregistering events, etc.
-- Expected result: Events are unregistered, frame is hidden if it exists and plug-ins into other mods are removed
-- Input: None
-- Output: None

function addon:OnDisable()

	-- If we disable the addon when the GUI is up, hide it.
	if (addon.Frame) then
		addon.Frame:Hide()
	end

	-- Remove the option from Manufac
	if Manufac then
		Manufac.options.args.ARLScan = nil
	end

end

-- Description: Shows the scan button when the trade skill window is open.
-- Expected result: Scan button is shown.
-- Input: None
-- Output: None

--[[

	Event handling functions

]]--

function addon:TRADE_SKILL_SHOW()

	addon.SkillType = "Trade"

	if (addon.ScanButton and not Skillet) then
		self:ShowScanButton()
	end

end

-- Description: Hides the scan button when the trade skill window is closed.
-- Expected result: Scan button is hidden and the GUI is closed if the person has that option selected.
-- Input: None
-- Output: None

function addon:TRADE_SKILL_CLOSE()

	addon.SkillType = nil

	if (addon.db.profile.closeguionskillclose and addon.Frame) then
		self:CloseWindow()
	end

	if (addon.ScanButton and not Skillet) then
		addon.ScanButton:Hide()
	end

end

-- Description: Will update the internal list of faction tables when a players faction changes
-- Expected result: Faction table is updated with new reputation levels
-- Input: None
-- Output: None

function addon:UPDATE_FACTION()

	-- Reputation has changed so lets update the table
	self:SetRepDB()

end

--[[

	Player Data Acquisition Functions

]]--

-- Description: Scans all reputations to get reputation levels to determine if the player can learn a reputation recipe
-- Expected result: Updates a table with players current faction level
-- Input: Faction table
-- Output: None, array is referenced

function addon:GetFactionLevels(RepTable)
-- Bug here when I reload UI
	if (not RepTable) then
		return
	end

	for i=1,GetNumFactions(),1 do
		local name,_,replevel = GetFactionInfo(i)

		-- If the rep is greater than neutral
		if (replevel > 4) then
		-- We use levels of 0, 1, 2, 3, 4 internally for reputation levels, make it corrospond here
			RepTable[name] = replevel - 4
		end
	end
end

-- Description: Scans first 25 spellbook slots to identify all applicable professions
-- Expected result: Updates a table with players current professions
-- Input: Profession table
-- Output: None, array is referenced

function addon:GetKnownProfessions(ProfTable)

	-- Reset the table, they may have unlearnt a profession
	for i in pairs(ProfTable) do

		ProfTable[i] = false

	end

	-- Scan through the spell book getting the spell names
	for index=1,25,1 do

		local spellName = GetSpellName(index, BOOKTYPE_SPELL)

		if (not spellName) or (index == 25) then

			-- Nothing found
			break

		end

		if (ProfTable[spellName] == false) then

			ProfTable[spellName] = true

		end
	end

end

-- Description: Scans first 25 spellbook slots to identify which trade skill speciality we have
-- Expected result: Returns the name of the trade skill speciality
-- Input: Specialty Table and Current Profession
-- Output: String of current speciality

function addon:GetTradeSpeciality(SpecialtyTable, playerdata)

	--Scan the first 25 entries
	for index=1,25,1 do

		local spellName = GetSpellName(index, BOOKTYPE_SPELL)

		-- Nothing found, return nothing
		if (not spellName) or (index == 25) or (not SpecialtyTable[playerdata.playerProfession]) then

			return ""

		-- We have a match, return that spell name
		elseif (SpecialtyTable[playerdata.playerProfession][spellName]) then

			return spellName

		end

	end

end

--[[
	Tradeskill functions
--]]

-- Description: Adds a specific recipe, along with it's info to an array
-- Expected result: The RecipeDB will have a recipe entry created for it.
-- Input: Recipe array reference, SpellID of recipe, Skill level of recipe, Item ID of item created, Rarity of recipe, and speciality of the recipe
-- Output: None, array is passed as a reference

function addon:addTradeSkill(RecipeDB, SpellID, SkillLevel, ItemID, Rarity, Speciality)

	--[[ 
		Recipe DB Structures are defined in Documentation.lua
	--]]
	
	-- Creates a table in the RecipeListing table storing all information about a recipe
	RecipeDB[SpellID] = {}
	-- Set the information passed
	RecipeDB[SpellID]["Level"] = SkillLevel
	RecipeDB[SpellID]["ItemID"] = ItemID
	RecipeDB[SpellID]["Rarity"] = Rarity

	-- Get the recipe link from the spell ID
	RecipeDB[SpellID]["RecipeLink"] = GetSpellLink(SpellID) or nil

	-- Get the recipe name now
	RecipeDB[SpellID]["Name"] = GetSpellInfo(SpellID) or nil

	if (RecipeDB[SpellID]["Name"] == nil) then
		self:Print(format(L["SpellIDCache"],SpellID))
	end

	-- All recipes are unknown until scan occurs
	RecipeDB[SpellID]["Known"] = false

	-- All recipes are set to be displayed until the filtering occurs
	RecipeDB[SpellID]["Display"] = true

	-- All recipes are set to be showing in the search results
	RecipeDB[SpellID]["Search"] = true

	-- Create the flag space in the RecipeDB
	RecipeDB[SpellID]["Flags"] = {}

	-- Set all the flags to be false, will also set the padding spaces to false as well.
	for i=1,addon.MaxFilterIndex,1 do

		RecipeDB[SpellID]["Flags"][tonumber(i)] = false

	end

	-- Create the Acquire space in the RecipeDB
	RecipeDB[SpellID]["Acquire"] = {}

	RecipeDB[SpellID]["Speciality"] = Speciality or nil

end

-- Description: Adds all flag related information to the RecipeDB associated with the spell ID
-- Expected result: The RecipeDB will have flag information added for a specific recipe
-- Input: Recipe array reference, SpellID of recipe, all flags
-- Output: None, array is passed as a reference

function addon:addTradeFlags(RecipeDB, SpellID, ...)

	-- flags are defined in Documentation.lua

	local numvars = select('#',...)

	-- Find out how many flags we're adding
	for i=1,numvars,1 do

		-- Get the value of the current flag
		local flag = select(i, ...)

		RecipeDB[SpellID]["Flags"][flag] = true

	end
	

end

-- Description: Adds all Acquire related information to the RecipeDB associated with the spell ID
-- Expected result: The RecipeDB will have acqusition information added for a specific recipe
-- Input: Recipe array reference, SpellID of recipe, all Acquire info
-- Output: None, array is passed as a reference

function addon:addTradeAcquire(RecipeDB, SpellID, ...)

	-- Find out how many flags we're adding
	local numvars = select('#',...)

	-- Index for the number of Acquire entries we have
	local index = 1

	-- Index for which variables we're parsing through
	local i = 1

	while (i < numvars) do
		-- Create the space for the current Acquire method
		RecipeDB[SpellID]["Acquire"][index] = {}

		-- Get the Type and ID of the values
		local AcquireType, AcquireID = select(i, ...)

		RecipeDB[SpellID]["Acquire"][index]["Type"] = AcquireType
		RecipeDB[SpellID]["Acquire"][index]["ID"] = AcquireID

		i = i + 2

		if (AcquireType == 6) then
			local RepLevel, RepVendor = select(i, ...)
			RecipeDB[SpellID]["Acquire"][index]["RepLevel"] = RepLevel
			RecipeDB[SpellID]["Acquire"][index]["RepVendor"] = RepVendor
			i = i + 2
		end

		index = index + 1
	end
end

-- Description: Adds a specific entry (ie: vendor, mob, etc) to the lookup list
-- Expected result: Look up list will have an entry added to it
-- Input: List to add to, ID of entry, Name, Faction Location and Coordinates (X and Y)
-- Output: None, array is passed as a reference

function addon:addLookupList(DB, ID, Name, Loc, Coordx, Coordy, Faction)

	--[[
		For individual database structures, see Documentation.lua
	]]--

	DB[ID] = {}
	DB[ID]["Name"] = Name

	if (Loc) then

		DB[ID]["Location"] = Loc

	end

	if (Coordx) and (Coordy) then

		DB[ID]["Coordx"] = Coordx
		DB[ID]["Coordy"] = Coordy

	end

	if (Faction) then

		if (Faction == 1) then

			DB[ID]["Faction"] = BFAC["Horde"]

		elseif (Faction == 2) then

			DB[ID]["Faction"] = BFAC["Alliance"]

		elseif (Faction == 3) then

			DB[ID]["Faction"] = BFAC["Neutral"]

		end

	end

end

--[[

	Recipe Scanning Functions

]]--

-- Description: Scans the recipe listing and marks known recipes as true in the database
-- Expected result: The array of Recipes will have all known recipes toggles to true
-- Input: Recipe Array
-- Output: The total number of recipes known

function addon:ScanForKnownRecipes(RecipeDB)

	self:Print("DEBUG: Scanning for known recipes.")

	-- Clear the "Have Materials" check box
	if not Skillet and TradeSkillFrameAvailableFilterCheckButton:GetChecked() then
		TradeSkillFrameAvailableFilterCheckButton:SetChecked(false)
		TradeSkillOnlyShowMakeable(false)
	end

	-- Clear the sub-classes filters
	SetTradeSkillSubClassFilter(0, 1, 1)
	UIDropDownMenu_SetSelectedID(TradeSkillSubClassDropDown, 1)

	-- Clear the inventory slot filter
	SetTradeSkillInvSlotFilter(0, 1, 1);
	UIDropDownMenu_SetSelectedID(TradeSkillInvSlotDropDown, 1)

	-- Expand all headers so we can see all the recipes there are
	for i = GetNumTradeSkills(), 1, -1 do
		local _, tradeType = GetTradeSkillInfo(i)
		if tradeType == "header" then
			ExpandTradeSkillSubClass(i)
		end
	end

	local RecipeCount = 0

	-- Scan through all recipes
	for i = 1, GetNumTradeSkills() do

		local tradeName, tradeType = GetTradeSkillInfo(i)

		-- Ignore all trade skill headers
		if (tradeType ~= "header") then

			-- Get the trade skill link for the specified recipe
			local SpellLink = GetTradeSkillRecipeLink(i)
			-- Get the SpellID from the spell link
			local _,_,SpellString = string.find(SpellLink, "enchant:(%d+)")
			local SpellID = tonumber(SpellString)

			-- Spell ID is in RecipeDB so lets flag it as known
			if (RecipeDB[SpellID]) then
				-- Update array that recipe was found
				RecipeDB[SpellID]["Known"] = true
				RecipeCount = RecipeCount + 1
			-- We didn't find it in our database, lets notify people that we don't have it
			else
				self:Print(self:Red(tradeName .. " " .. SpellString) .. self:White(L["MissingFromDB"]))	
			end

		end

	end

	return RecipeCount

end

-- Description: Scans a specific recpie to determine if it is to be displayed or not.
-- Expected result: Returns true or false depending on if the recipe should be displayed
-- Input: 
-- Output: Boolean on whether or not to display the recipe

function addon:CheckDisplayRecipe(Recipe, AllSpecialitiesTable, playerProfessionLevel, playerProfession, playerSpeciality, playerFaction, playerClass)

	-- For flag info see comments at start of file in comments

	return true

end

-- Description: Creates an array of which factions we want to include in our display and which ones to ignore
-- Expected result: Array of all factions created with those which we want to display having a true flag
-- Input: Rep Table Array
-- Output: None, array is passed as a reference

function addon:PopulateRepFilters(RepTable)

	self:Print("DEBUG: Updating Rep Filters")
	local repfilters = addon.db.profile.filters.rep
	RepTable[BFAC["The Scryers"]] = repfilters.scryer
	RepTable[BFAC["The Aldor"]] = repfilters.aldor
	RepTable[BFAC["Argent Dawn"]] = repfilters.argentdawn
	RepTable[BFAC["Ashtongue Deathsworn"]] = repfilters.ashtonguedeathsworn
	RepTable[BFAC["Cenarion Circle"]] = repfilters.cenarioncircle
	RepTable[BFAC["Cenarion Expedition"]] = repfilters.cenarionexpedition
	RepTable[BFAC["The Consortium"]] = repfilters.consortium
	RepTable[BFAC["Honor Hold"]] = repfilters.hellfire
	RepTable[BFAC["Thrallmar"]] = repfilters.hellfire
	RepTable[BFAC["Keepers of Time"]] = repfilters.keepersoftime
	RepTable[BFAC["Kurenai"]] = repfilters.nagrand
	RepTable[BFAC["The Mag'har"]] = repfilters.nagrand
	RepTable[BFAC["Lower City"]] = repfilters.lowercity
	RepTable[BFAC["The Scale of the Sands"]] = repfilters.scaleofthesands
	RepTable[BFAC["The Sha'tar"]] = repfilters.shatar
	RepTable[BFAC["Shattered Sun Offensive"]] = repfilters.shatteredsun
	RepTable[BFAC["Sporeggar"]] = repfilters.sporeggar
	RepTable[BFAC["Thorium Brotherhood"]] = repfilters.thoriumbrotherhood
	RepTable[BFAC["Timbermaw Hold"]] = repfilters.timbermaw
	RepTable[BFAC["The Violet Eye"]] = repfilters.violeteye
	RepTable[BFAC["Zandalar Tribe"]] = repfilters.zandalar
-- Below this is WotLK data. May need to be changed post beta.
	RepTable[BFAC["Argent Crusade"]] = repfilters.argentcrusade
	RepTable[BFAC["Frenzyheart Tribe"]] = repfilters.frenzyheart
	RepTable[BFAC["Knights of the Ebon Blade"]] = repfilters.ebonblade
	RepTable[BFAC["Kirin Tor"]] = repfilters.kirintor
	RepTable[BFAC["The Sons of Hodir"]] = repfilters.sonsofhodir
	RepTable[BFAC["The Kalu'ak"]] = repfilters.kaluak
	RepTable[BFAC["The Oracles"]] = repfilters.oracles
	RepTable[BFAC["The Wyrmrest Accord"]] = repfilters.wyrmrest
	RepTable[BFAC["The Silver Convenant"]] = repfilters.silvercovenant
	RepTable[BFAC["The Sunreavers"]] = repfilters.sunreavers
	RepTable[BFAC["Explorer's League"]] = repfilters.explorersleague
	RepTable[BFAC["Valiance Expedition"]] = repfilters.valiance
	RepTable[BFAC["The Hand of Vengeance"]] = repfilters.handofvengeance
	RepTable[BFAC["The Taunka"]] = repfilters.taunka
	RepTable[BFAC["Warsong Offensive"]] = repfilters.warsongoffensive
end

-- Description: Scans the recipe listing and updates the filters according to user preferences
-- Expected result: The array of Recipes will have all Display flags toggles according to display preferences and the number of filtered recipes
-- Input: Recipe Array, Skill level for current profession, name of current profession, and current profession speciality
-- Output: Number of recipes that are filtered

function addon:UpdateFilters(RecipeDB, AllSpecialitiesTable, playerdata)

	local playerProfessionLevel = playerdata.playerProfessionLevel
	local playerProfession = playerdata.playerProfession
	local playerSpeciality = playerdata.playerSpeciality
	local playerFaction = playerdata.playerFaction
	local playerClass = playerdata.playerClass

	local FilteredRecipes = 0

	-- Parse through all the entries in the Recipe array
	for RecipeID in pairs(RecipeDB) do

		-- Determine if we are to display this recipe or not
		local displayflag = self:CheckDisplayRecipe(RecipeDB[RecipeID], AllSpecialitiesTable, playerProfessionLevel, playerProfession, playerSpeciality, playerFaction, playerClass)

		if (displayflag == false) then

			FilteredRecipes = FilteredRecipes + 1

		end

		-- Set the display flag
		RecipeDB[RecipeID]["Display"] = displayflag

	end

	return FilteredRecipes

end

--[[

	ARL Logic Functions

]]--

-- Description: Determines which profession we are dealing with and loads up the recipe information for it.
-- Expected result: The RecipeDBis populated with recipe information
-- Input: RecipeDB, Current Profession we're examining
-- Output: Total number of recipes in the database

local function InitializeRecipes(RecipeDB, playerdata)

	local recipecount = 0

	-- Table of all possible professions with init functions
	local professiontable =
	{
		[GetSpellInfo(2259)] = addon.InitAlchemy,
		[GetSpellInfo(2018)] = addon.InitBlacksmithing,
		[GetSpellInfo(2550)] = addon.InitCooking,
		[GetSpellInfo(7411)] = addon.InitEnchanting,
		[GetSpellInfo(4036)] = addon.InitEngineering,
		-- Use first aid spell of applying bandages to fix issues with other localizations
		[GetSpellInfo(746)] = addon.InitFirstAid,
		-- Hack to get first aid working on frFR since I can't seem to get a proper spell ID :P
		["Premiers soins"] = addon.InitFirstAid,
		--[GetSpellInfo(3273)] = addon.InitFirstAid,
		[GetSpellInfo(2108)] = addon.InitLeatherworking,
		[GetSpellInfo(2575)] = addon.InitSmelting,
		[GetSpellInfo(3908)] = addon.InitTailoring,
		[GetSpellInfo(25229)] = addon.InitJewelcrafting,
		[GetSpellInfo(45357)] = addon.InitInscription,
		[GetSpellInfo(28481)] = addon.InitRuneforging,
	}

	-- Thanks to sylvanaar/xinhuan for the code snippet
	local a = professiontable[playerdata.playerProfession]

	if a then

		recipecount = a(addon, RecipeDB)

	else

		addon:Print(L["UnknownTradeSkill"]:format(playerdata.playerProfession))

	end

	return recipecount

end

-- Description: Determines what to do when the slash command is called.
-- Expected result: The appropiate window is opened based on the slash command.
-- Input: None
-- Output: None

function addon:ChatCommand(input)
	InterfaceOptionsFrame_OpenToFrame( self.optionsFrame.AckisRecipeList )
--[[
	if (not input) or (input and input:trim() == "") or (input == string.lower(L["About"])) then
		-- Open About panel if there's no parameters
		InterfaceOptionsFrame_OpenToFrame(self.optionsFrame["about"])
	elseif (input == string.lower(L["Sort"])) then
		InterfaceOptionsFrame_OpenToFrame(self.optionsFrame[L["Sort"] ])
	elseif (input == string.lower(L["Filter"])) then
		InterfaceOptionsFrame_OpenToFrame(self.optionsFrame[L["Filter"] ])
	elseif (input == string.lower(L["Display"])) then
		InterfaceOptionsFrame_OpenToFrame(self.optionsFrame["display"])
	elseif (input == string.lower(L["Profile"])) then
		InterfaceOptionsFrame_OpenToFrame(self.optionsFrame["profiles"])
	else
		LibStub("AceConfigCmd-3.0"):HandleCommand("arl", "Ackis Recipe List", input)
	end
]]--
end

do

	local RecipeList = nil

	local MobList = nil
	local QuestList = nil
	local ReputationList = nil
	local TrainerList = nil
	local SeasonalList = nil
	local VendorList = nil
	local RepFilters = nil
	local AllSpecialitiesTable = nil

	local playerdata = nil

	-- Description: Updates the reputation table.  This only happens seldomly so I'm not worried about effeciency
	-- Expected result: Reputation table is updated with appropiate levels
	-- Input: None
	-- Output: None

	function addon:SetRepDB()

		if (playerdata and playerdata["Reputation"]) then

			self:GetFactionLevels(playerdata["Reputation"])

		end

	end

	-- Description: Function called when the scan button is clicked.   Parses recipes and displays output
	-- Expected result: A gui window of all recipes accoring to filter is printed out
	-- Input: None
	-- Output: None

	function addon:AckisRecipeList_Command(textdump)

		local FilteredRecipes = 0
		local FoundRecipes = 0
		local TotalRecipes = 0

		-- If we don't have a trade skill window open, lets return out of here
		if (addon.SkillType == nil) then

			self:Print(L["OpenTradeSkillWindow"])
			return

		-- Trade type skills
		elseif (addon.SkillType == "Trade") then

			-- First time a scan has been run, we need to get the player specifc data, specifically faction information, profession information and other pertinant data.
			if (playerdata == nil) then

				playerdata = {}

				playerdata.playerFaction = UnitFactionGroup("player")
				_, playerdata.playerClass = UnitClass("player")

				playerdata["Reputation"] = {}
				self:GetFactionLevels(playerdata["Reputation"])

				playerdata["Professions"] = {
					[GetSpellInfo(2259)] = false, -- Alchemy
					[GetSpellInfo(2018)] = false, -- Blacksmithing
					[GetSpellInfo(2550)] = false, -- Cooking
					[GetSpellInfo(7411)] = false, -- Enchanting
					[GetSpellInfo(4036)] = false, -- Engineering
					[GetSpellInfo(746)] = false, -- First Aid
					[GetSpellInfo(2108)] = false, -- Leatherworking
					[GetSpellInfo(2575)] = false, -- Smelting
					[GetSpellInfo(3908)] = false, -- Tailoring
					[GetSpellInfo(25229)] = false, -- Jewelcrafting
					[GetSpellInfo(45357)] = false, -- Inscription
					[GetSpellInfo(28481)] = false, -- Runeforging
				}

				self:GetKnownProfessions(playerdata["Professions"])

				-- All Alchemy Specialities
				local AlchemySpec = {
					[GetSpellInfo(28674)] = true,
					[GetSpellInfo(28678)] = true,
					[GetSpellInfo(28676)] = true,
				}

				-- All Blacksmithing Specialities
				local BlacksmithSpec = {
					[GetSpellInfo(9788)] = true, -- Armorsmith
					[GetSpellInfo(17041)] = true, -- Master Axesmith
					[GetSpellInfo(17040)] = true, -- Master Hammersmith
					[GetSpellInfo(17039)] = true, -- Master Swordsmith
					[GetSpellInfo(9787)] = true, -- Weaponsmith
				}

				-- All Engineering Specialities
				local EngineeringSpec = {
					[GetSpellInfo(20219)] = true, -- Gnomish
					[GetSpellInfo(20222)] = true, -- Goblin
				}

				-- All Leatherworking Specialities
				local LeatherworkSpec = {
					[GetSpellInfo(10657)] = true, -- Dragonscale
					[GetSpellInfo(10659)] = true, -- Elemental
					[GetSpellInfo(10661)] = true, -- Tribal
				}

				-- All Tailoring Specialities
				local TailorSpec = {
					[GetSpellInfo(26797)] = true, -- Spellfire
					[GetSpellInfo(26801)] = true, -- Shadoweave
					[GetSpellInfo(26798)] = true, -- Primal Mooncloth
				}

				-- List of classes which have specialities
				local SpecialtyTable = {
					[GetSpellInfo(2259)] = AlchemySpec,
					[GetSpellInfo(2018)] = BlacksmithSpec,
					[GetSpellInfo(4036)] = EngineeringSpec,
					[GetSpellInfo(2108)] = LeatherworkSpec,
					[GetSpellInfo(3908)] = TailorSpec,
				}

				-- List containing all possible specialities
				AllSpecialitiesTable = {}

				-- Populate the speciality table with all specialities, not adding alchemy because no recipes have alchemy filters
				for i in pairs(BlacksmithSpec) do AllSpecialitiesTable[i] = true end
				for i in pairs(EngineeringSpec) do AllSpecialitiesTable[i] = true end
				for i in pairs(LeatherworkSpec) do AllSpecialitiesTable[i] = true end
				for i in pairs(TailorSpec) do AllSpecialitiesTable[i] = true end

			end

			-- Initializes the mob list
			if (MobList == nil) then

				MobList = {}
				self:InitMob(MobList)

			end

			-- Initializes the quest list
			if (QuestList == nil) then

				QuestList = {}
				self:InitQuest(QuestList)

			end

			-- Initializes the reputation list
			if (ReputationList == nil) then

				ReputationList = {}
				self:InitReputation(ReputationList)

			end

			-- Initializes the trainer list
			if (TrainerList == nil) then

				TrainerList = {}
				self:InitTrainer(TrainerList)

			end

			-- Initializes the season list
			if (SeasonalList == nil) then

				SeasonalList = {}
				self:InitSeasons(SeasonalList)

			end

			-- Initializes the vendor list
			if (VendorList == nil) then

				VendorList = {}
				self:InitVendor(VendorList)

			end

			-- Initializes the reputation filters
			-- Don't assign values no because we do a scan later on
			if (RepFilters == nil) then

				RepFilters = {}

			end

			-- Initializes the recipe list
			-- Note there is no reset function implemented yet
			if (RecipeList == nil) then

				RecipeList = {}

			end

			-- Get the name of the current trade skill opened, along with the current level of the skill.
			playerdata.playerProfession, playerdata.playerProfessionLevel = GetTradeSkillLine()

			-- Get the current profession speciality
			playerdata.playerSpeciality = self:GetTradeSpeciality(AllSpecialitiesTable, playerdata)

			-- Add the recipes to the database
			playerdata.totalRecipes = InitializeRecipes(RecipeList, playerdata)

			-- Scan all recipes and mark the ones which ones we know
			playerdata.foundRecipes = self:ScanForKnownRecipes(RecipeList)

			-- Update the table containing which reps to display
			self:PopulateRepFilters(RepFilters)

			-- Add filtering flags to the recipes
			playerdata.filteredRecipes = self:UpdateFilters(RecipeList, AllSpecialitiesTable, playerdata)

		end

		-- Sort the recipe list now
		local sortedindex = self:SortMissingRecipes(RecipeList)

		if (textdump == true) then

			local temptext = self:GetTextDump(RecipeList)

			self:DisplayTextDump(temptext)

		else

			self:CreateFrame( RecipeList, sortedindex, playerdata, TrainerList,
									VendorList, QuestList, ReputationList, SeasonalList, MobList )

		end

	end

end

--[[

	Sorting Functions

]]--

-- Description: Sorts the recipe Database depending on the settings defined in the database.
-- Expected result: A sorted array indexing values in the RecipeDB is returned.
-- Input: The Recipe Database
-- Output: A pointer to an array containing sorted values

function addon:SortMissingRecipes(RecipeDB)

	self:Print("DEBUG: Sorting recipe array")

	-- Create a new array for the sorted index
	local SortedRecipeIndex = {}

	-- Find out how he want to sort
	local sorttype = addon.db.profile.sorting

	-- Get all the indexes of the RecipeListing
	for n, v in pairs(RecipeDB) do
		table.insert(SortedRecipeIndex, n)
	end

	if (sorttype == L["Skill"]) then

		table.sort(SortedRecipeIndex, function(a,b) return RecipeDB[a]["Level"] < RecipeDB[b]["Level"] end)

	elseif (sorttype == L["Name"]) then

		table.sort(SortedRecipeIndex, function(a,b) return RecipeDB[a]["Name"] < RecipeDB[b]["Name"] end)

	elseif (sorttype == L["Acquisition"]) then

		table.sort(SortedRecipeIndex, function(a,b) return RecipeDB[a]["Acquire"][1]["Type"] < RecipeDB[b]["Acquire"][1]["Type"] end)

	end

	return SortedRecipeIndex

end

--[[

	Searching Functions

]]--

-- Description: Scans through the recipe database and toggles the flag on if the item is in the search criteria
-- Expected result: The Search flag is toggle off for anything that does not meet search criteria
-- Input: Recipe database
-- Output: None, array is a reference

function addon:SearchRecipeDB(RecipeDB, searchstring)

	-- Go through the entire database
	for SpellID in pairs(RecipeDB) do

		-- Allow us to search by spell ID
		if string.find(SpellID,searchstring) or
		-- Allow us to search byitem ID
		string.find(RecipeDB[SpellID]["ItemID"],searchstring) or
		-- Allow us to search by name
		string.find(RecipeDB[SpellID]["Name"],searchstring) then
		-- Allow us to search in the Acquire information
		--string.find(RecipeDB[SpellID]["Name"],searchstring) then

			RecipeDB[SpellID]["Search"] = true

		else

			RecipeDB[SpellID]["Search"] = false

		end

	end

end

-- Description: Goes through the recipe database and resets all the search flags
-- Expected result: All search flags are reset to true
-- Input: Recipe database
-- Output: None, array is a reference

function addon:ResetSearch(RecipeDB)

	for SpellID in pairs(RecipeDB) do

		RecipeDB[SpellID]["Search"] = true

	end

end

--[[

	Text dumping functions

]]--

-- Description: Scans through the recipe database providing a string of comma seperated values for all recipe information
-- Expected result: Single string of all recipes provided.
-- Input: Recipe database
-- Output: Array is a reference, string of text dump

function addon:GetTextDump(RecipeDB)

	local texttable = {}

	for SpellID in pairs(RecipeDB) do

		if (RecipeDB[SpellID]["Known"]) then

			table.insert(texttable,SpellID .. "," .. RecipeDB[SpellID]["Name"] .. "," .. RecipeDB[SpellID]["Level"] .. "," .. ",true")

		else

			table.insert(texttable,SpellID .. "," .. RecipeDB[SpellID]["Name"] .. "," .. RecipeDB[SpellID]["Level"] .. "," .. ",false")

		end

	end

	return table.concat(texttable,"\n")

end
